@@ -1,13 +1,13 @@
 /**
- * 
+ *
  * Ulfius Framework
- * 
+ *
  * REST framework library
- * 
+ *
  * u_umap.c: Simple map structure functions definitions
  * not memory friendly, all pointer returned must be freed after use
- * 
- * Copyright 2015-2017 Nicolas Mora <mail@babelouest.org>
+ *
+ * Copyright 2015-2020 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -21,7 +21,7 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
- * 
+ *
  */
 
 #include <stdio.h>
@@ -30,37 +30,32 @@
 #include "u_private.h"
 #include "ulfius.h"
 
-/**
- * initialize a struct _u_map
- * this function MUST be called after a declaration or allocation
- * return U_OK on success
- */
-int u_map_init(struct _u_map * map) {
-  if (map != NULL) {
-    map->nb_values = 0;
-    map->keys = o_malloc(sizeof(char *));
-    if (map->keys == NULL) {
-      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for map->keys");
+int u_map_init(struct _u_map * u_map) {
+  if (u_map != NULL) {
+    u_map->nb_values = 0;
+    u_map->keys = o_malloc(sizeof(char *));
+    if (u_map->keys == NULL) {
+      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for u_map->keys");
       return U_ERROR_MEMORY;
     }
-    map->keys[0] = NULL;
+    u_map->keys[0] = NULL;
 
-    map->values = o_malloc(sizeof(char *));
-    if (map->values == NULL) {
-      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for map->values");
-      o_free(map->keys);
+    u_map->values = o_malloc(sizeof(char *));
+    if (u_map->values == NULL) {
+      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for u_map->values");
+      o_free(u_map->keys);
       return U_ERROR_MEMORY;
     }
-    map->values[0] = NULL;
-    
-    map->lengths = o_malloc(sizeof(size_t));
-    if (map->lengths == NULL) {
-      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for map->lengths");
-      o_free(map->keys);
-      o_free(map->values);
+    u_map->values[0] = NULL;
+
+    u_map->lengths = o_malloc(sizeof(size_t));
+    if (u_map->lengths == NULL) {
+      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for u_map->lengths");
+      o_free(u_map->keys);
+      o_free(u_map->values);
       return U_ERROR_MEMORY;
     }
-    map->lengths[0] = 0;
+    u_map->lengths[0] = 0;
 
     return U_OK;
   } else {
@@ -68,14 +63,15 @@ int u_map_init(struct _u_map * map) {
   }
 }
 
-/**
- * free the struct _u_map's inner components
- * return U_OK on success
- */
 int u_map_clean(struct _u_map * u_map) {
+  int i;
   if (u_map != NULL) {
-    u_map_clean_enum(u_map->keys);
-    u_map_clean_enum(u_map->values);
+    for (i=0; i<u_map->nb_values; i++) {
+      o_free(u_map->keys[i]);
+      o_free(u_map->values[i]);
+    }
+    o_free(u_map->keys);
+    o_free(u_map->values);
     o_free(u_map->lengths);
     return U_OK;
   } else {
@@ -83,10 +79,6 @@ int u_map_clean(struct _u_map * u_map) {
   }
 }
 
-/**
- * free the struct _u_map and its components
- * return U_OK on success
- */
 int u_map_clean_full(struct _u_map * u_map) {
   if (u_map_clean(u_map) == U_OK) {
     o_free(u_map);
@@ -96,10 +88,6 @@ int u_map_clean_full(struct _u_map * u_map) {
   }
 }
 
-/**
- * free an enum return by functions u_map_enum_keys or u_map_enum_values
- * return U_OK on success
- */
 int u_map_clean_enum(char ** array) {
   int i;
   if (array != NULL) {
@@ -114,27 +102,14 @@ int u_map_clean_enum(char ** array) {
   }
 }
 
-/**
- * returns an array containing all the keys in the struct _u_map
- * return an array of char * ending with a NULL element
- */
 const char ** u_map_enum_keys(const struct _u_map * u_map) {
   return (const char **)u_map->keys;
 }
 
-/**
- * returns an array containing all the values in the struct _u_map
- * return an array of char * ending with a NULL element
- */
 const char ** u_map_enum_values(const struct _u_map * u_map) {
   return (const char **)u_map->values;
 }
 
-/**
- * return true if the sprcified u_map contains the specified key
- * false otherwise
- * search is case sensitive
- */
 int u_map_has_key(const struct _u_map * u_map, const char * key) {
   int i;
   if (u_map != NULL && key != NULL) {
@@ -147,20 +122,10 @@ int u_map_has_key(const struct _u_map * u_map, const char * key) {
   return 0;
 }
 
-/**
- * return true if the sprcified u_map contains the specified value
- * false otherwise
- * search is case sensitive
- */
 int u_map_has_value(const struct _u_map * u_map, const char * value) {
   return u_map_has_value_binary(u_map, value, o_strlen(value));
 }
 
-/**
- * return true if the sprcified u_map contains the specified value
- * false otherwise
- * search is case sensitive
- */
 int u_map_has_value_binary(const struct _u_map * u_map, const char * value, size_t length) {
   int i;
   if (u_map != NULL && value != NULL) {
@@ -173,11 +138,6 @@ int u_map_has_value_binary(const struct _u_map * u_map, const char * value, size
   return 0;
 }
 
-/**
- * add the specified key/value pair into the specified u_map
- * if the u_map already contains a pair with the same key, replace the value
- * return U_OK on success
- */
 int u_map_put(struct _u_map * u_map, const char * key, const char * value) {
   if (value != NULL) {
     return u_map_put_binary(u_map, key, value, 0, o_strlen(value)+1);
@@ -186,12 +146,6 @@ int u_map_put(struct _u_map * u_map, const char * key, const char * value) {
   }
 }
 
-/**
- * add the specified key/binary value pair into the specified u_map
- * if the u_map already contains a pair with the same key,
- * replace the value at the specified offset with the specified length
- * return U_OK on success
- */
 int u_map_put_binary(struct _u_map * u_map, const char * key, const char * value, uint64_t offset, size_t length) {
   int i;
   char * dup_key, * dup_value;
@@ -208,9 +162,13 @@ int u_map_put_binary(struct _u_map * u_map, const char * key, const char * value
         }
         if (value != NULL) {
           memcpy(u_map->values[i]+offset, value, length);
-        }
-        if (u_map->lengths[i] < (offset + length)) {
-          u_map->lengths[i] = (offset + length);
+          if (u_map->lengths[i] < (offset + length)) {
+            u_map->lengths[i] = (offset + length);
+          }
+        } else {
+          o_free(u_map->values[i]);
+          u_map->values[i] = o_strdup("");
+          u_map->lengths[i] = 0;
         }
         return U_OK;
       }
@@ -231,9 +189,9 @@ int u_map_put_binary(struct _u_map * u_map, const char * key, const char * value
         }
         memcpy((dup_value + offset), value, length);
       } else {
-        dup_value = NULL;
+        dup_value = o_strdup("");
       }
-      
+
       // Append key
       for (i = 0; u_map->keys[i] != NULL; i++);
       u_map->keys = o_realloc(u_map->keys, (i + 2)*sizeof(char *));
@@ -245,7 +203,7 @@ int u_map_put_binary(struct _u_map * u_map, const char * key, const char * value
       }
       u_map->keys[i] = (char *)dup_key;
       u_map->keys[i+1] = NULL;
-      
+
       // Append value
       u_map->values = o_realloc(u_map->values, (i + 2)*sizeof(char *));
       if (u_map->values == NULL) {
@@ -256,7 +214,7 @@ int u_map_put_binary(struct _u_map * u_map, const char * key, const char * value
       }
       u_map->values[i] = (char *)dup_value;
       u_map->values[i+1] = NULL;
-      
+
       // Append length
       u_map->lengths = o_realloc(u_map->lengths, (i + 2)*sizeof(size_t));
       if (u_map->lengths == NULL) {
@@ -267,7 +225,7 @@ int u_map_put_binary(struct _u_map * u_map, const char * key, const char * value
       }
       u_map->lengths[i] = (offset + length);
       u_map->lengths[i+1] = 0;
-      
+
       u_map->nb_values++;
     }
     return U_OK;
@@ -276,13 +234,9 @@ int u_map_put_binary(struct _u_map * u_map, const char * key, const char * value
   }
 }
 
-/**
- * remove an pair key/value that has the specified key
- * return U_OK on success, U_NOT_FOUND if key was not found, error otherwise
- */
 int u_map_remove_from_key(struct _u_map * u_map, const char * key) {
   int i, res, found = 0;
-  
+
   if (u_map == NULL || key == NULL) {
     return U_ERROR_PARAMS;
   } else {
@@ -303,13 +257,9 @@ int u_map_remove_from_key(struct _u_map * u_map, const char * key) {
   }
 }
 
-/**
- * remove all pairs key/value that has the specified key (case insensitive search)
- * return U_OK on success, U_NOT_FOUND if key was not found, error otherwise
- */
 int u_map_remove_from_key_case(struct _u_map * u_map, const char * key) {
   int i, res, found = 0;
-  
+
   if (u_map == NULL || key == NULL) {
     return U_ERROR_PARAMS;
   } else {
@@ -330,21 +280,13 @@ int u_map_remove_from_key_case(struct _u_map * u_map, const char * key) {
   }
 }
 
-/**
- * remove all pairs key/value that has the specified value
- * return U_OK on success, U_NOT_FOUND if key was not found, error otherwise
- */
 int u_map_remove_from_value(struct _u_map * u_map, const char * value) {
   return u_map_remove_from_value_binary(u_map, value, o_strlen(value));
 }
 
-/**
- * remove all pairs key/value that has the specified value up until the specified length
- * return U_OK on success, U_NOT_FOUND if key was not found, error otherwise
- */
 int u_map_remove_from_value_binary(struct _u_map * u_map, const char * value, size_t length) {
   int i, res, found = 0;
-  
+
   if (u_map == NULL || value == NULL) {
     return U_ERROR_PARAMS;
   } else {
@@ -365,13 +307,9 @@ int u_map_remove_from_value_binary(struct _u_map * u_map, const char * value, si
   }
 }
 
-/**
- * remove all pairs key/value that has the specified value (case insensitive search)
- * return U_OK on success, U_NOT_FOUND if key was not found, error otherwise
- */
 int u_map_remove_from_value_case(struct _u_map * u_map, const char * value) {
   int i, res, found = 0;
-  
+
   if (u_map == NULL || value == NULL) {
     return U_ERROR_PARAMS;
   } else {
@@ -392,10 +330,6 @@ int u_map_remove_from_value_case(struct _u_map * u_map, const char * value) {
   }
 }
 
-/**
- * remove the pair key/value at the specified index
- * return U_OK on success, U_NOT_FOUND if index is out of bound, error otherwise
- */
 int u_map_remove_at(struct _u_map * u_map, const int index) {
   int i;
   if (u_map == NULL || index < 0) {
@@ -425,17 +359,12 @@ int u_map_remove_at(struct _u_map * u_map, const int index) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for u_map->lengths");
       return U_ERROR_MEMORY;
     }
-    
+
     u_map->nb_values--;
     return U_OK;
   }
 }
 
-/**
- * get the value corresponding to the specified key in the u_map
- * return NULL if no match found
- * search is case sensitive
- */
 const char * u_map_get(const struct _u_map * u_map, const char * key) {
   int i;
   if (u_map != NULL && key != NULL) {
@@ -454,11 +383,6 @@ const char * u_map_get(const struct _u_map * u_map, const char * key) {
   }
 }
 
-/**
- * return true if the sprcified u_map contains the specified key
- * false otherwise
- * search is case insensitive
- */
 int u_map_has_key_case(const struct _u_map * u_map, const char * key) {
   int i;
   if (u_map != NULL && key != NULL) {
@@ -471,11 +395,6 @@ int u_map_has_key_case(const struct _u_map * u_map, const char * key) {
   return 0;
 }
 
-/**
- * return true if the sprcified u_map contains the specified value
- * false otherwise
- * search is case insensitive
- */
 int u_map_has_value_case(const struct _u_map * u_map, const char * value) {
   int i;
   if (u_map != NULL && value != NULL) {
@@ -488,11 +407,6 @@ int u_map_has_value_case(const struct _u_map * u_map, const char * value) {
   return 0;
 }
 
-/**
- * get the value corresponding to the specified key in the u_map
- * return NULL if no match found
- * search is case insensitive
- */
 const char * u_map_get_case(const struct _u_map * u_map, const char * key) {
   int i;
   if (u_map != NULL && key != NULL) {
@@ -507,11 +421,6 @@ const char * u_map_get_case(const struct _u_map * u_map, const char * key) {
   }
 }
 
-/**
- * get the value length corresponding to the specified key in the u_map
- * return -1 if no match found
- * search is case sensitive
- */
 ssize_t u_map_get_length(const struct _u_map * u_map, const char * key) {
   int i;
   if (u_map != NULL && key != NULL) {
@@ -526,11 +435,6 @@ ssize_t u_map_get_length(const struct _u_map * u_map, const char * key) {
   }
 }
 
-/**
- * get the value length corresponding to the specified key in the u_map
- * return -1 if no match found
- * search is case insensitive
- */
 ssize_t u_map_get_case_length(const struct _u_map * u_map, const char * key) {
   int i;
   if (u_map != NULL && key != NULL) {
@@ -545,11 +449,6 @@ ssize_t u_map_get_case_length(const struct _u_map * u_map, const char * key) {
   }
 }
 
-/**
- * Create an exact copy of the specified struct _u_map
- * return a reference to the copy, NULL otherwise
- * returned value must be cleaned after use
- */
 struct _u_map * u_map_copy(const struct _u_map * source) {
   struct _u_map * copy = NULL;
   const char ** keys, * value;
@@ -575,15 +474,10 @@ struct _u_map * u_map_copy(const struct _u_map * source) {
   return copy;
 }
 
-/**
- * Copy all key/values pairs of source into dest
- * If key is already present in dest, it's overwritten
- * return U_OK on success, error otherwise
- */
 int u_map_copy_into(struct _u_map * dest, const struct _u_map * source) {
   const char ** keys;
   int i, res;
-  
+
   if (source != NULL && dest != NULL) {
     keys = u_map_enum_keys(source);
     for (i=0; keys != NULL && keys[i] != NULL; i++) {
@@ -598,10 +492,6 @@ int u_map_copy_into(struct _u_map * dest, const struct _u_map * source) {
   }
 }
 
-/**
- * Return the number of key/values pair in the specified struct _u_map
- * Return -1 on error
- */
 int u_map_count(const struct _u_map * source) {
   if (source != NULL) {
     if (source->nb_values >= 0) {
@@ -611,10 +501,6 @@ int u_map_count(const struct _u_map * source) {
   return -1;
 }
 
-/**
- * Empty a struct u_map of all its elements
- * return U_OK on success, error otherwise
- */
 int u_map_empty(struct _u_map * u_map) {
   int ret = u_map_clean(u_map);
   if (ret == U_OK) {
