@@ -1,12 +1,12 @@
 /**
- * 
+ *
  * Ulfius Framework
- * 
+ *
  * REST framework library
- * 
+ *
  * u_response.c: response related functions defintions
- * 
- * Copyright 2015-2017 Nicolas Mora <mail@babelouest.org>
+ *
+ * Copyright 2015-2020 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -20,17 +20,13 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
- * 
+ *
  */
 #include <string.h>
 
 #include "u_private.h"
 #include "ulfius.h"
 
-/**
- * Add a cookie in the cookie map as defined in the RFC 6265
- * Returned value must be free'd after use
- */
 static char * ulfius_generate_cookie_header(const struct _u_cookie * cookie) {
   char * attr_expires = NULL, * attr_max_age = NULL, * attr_domain = NULL, * attr_path = NULL;
   char * attr_secure = NULL, * attr_http_only = NULL, * cookie_header_value = NULL, * same_site = NULL;
@@ -128,7 +124,7 @@ static char * ulfius_generate_cookie_header(const struct _u_cookie * cookie) {
     } else {
       same_site = o_strdup("");
     }
-    
+
     if (attr_expires == NULL || attr_max_age == NULL || attr_domain == NULL || attr_path == NULL || attr_secure == NULL || attr_http_only == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for ulfius_generate_cookie_header");
     } else {
@@ -157,11 +153,6 @@ static char * ulfius_generate_cookie_header(const struct _u_cookie * cookie) {
   }
 }
 
-/**
- * ulfius_set_response_header
- * adds headers defined in the response_map_header to the response
- * return the number of added headers, -1 on error
- */
 int ulfius_set_response_header(struct MHD_Response * response, const struct _u_map * response_map_header) {
   const char ** header_keys = u_map_enum_keys(response_map_header);
   const char * header_value;
@@ -181,11 +172,6 @@ int ulfius_set_response_header(struct MHD_Response * response, const struct _u_m
   return i;
 }
 
-/**
- * ulfius_set_response_cookie
- * adds cookies defined in the response_map_cookie
- * return the number of added headers, -1 on error
- */
 int ulfius_set_response_cookie(struct MHD_Response * mhd_response, const struct _u_response * response) {
   int ret;
   unsigned int i;
@@ -211,26 +197,12 @@ int ulfius_set_response_cookie(struct MHD_Response * mhd_response, const struct
   }
 }
 
-/**
- * ulfius_add_cookie_to_response
- * add a cookie to the cookie map
- * return U_OK on success
- */
-int ulfius_add_cookie_to_response(struct _u_response * response, const char * key, const char * value, const char * expires, const unsigned int max_age, 
+int ulfius_add_cookie_to_response(struct _u_response * response, const char * key, const char * value, const char * expires, const unsigned int max_age,
                                   const char * domain, const char * path, const int secure, const int http_only) {
   return ulfius_add_same_site_cookie_to_response(response, key, value, expires, max_age, domain, path, secure, http_only, U_COOKIE_SAME_SITE_NONE);
 }
 
-/**
- * ulfius_add_same_site_cookie_to_response
- * add a cookie to the cookie map with a SameSite attribute
- * the same_site parameter must have one of the following values:
- * - U_COOKIE_SAME_SITE_NONE   - No SameSite attribute
- * - U_COOKIE_SAME_SITE_STRICT - SameSite attribute set to 'Strict'
- * - U_COOKIE_SAME_SITE_LAX    - SameSite attribute set to 'Lax'
- * return U_OK on success
- */
-int ulfius_add_same_site_cookie_to_response(struct _u_response * response, const char * key, const char * value, const char * expires, const unsigned int max_age, 
+int ulfius_add_same_site_cookie_to_response(struct _u_response * response, const char * key, const char * value, const char * expires, const unsigned int max_age,
                                             const char * domain, const char * path, const int secure, const int http_only, const int same_site) {
   unsigned int i;
   if (response != NULL && key != NULL && (same_site == U_COOKIE_SAME_SITE_NONE || same_site == U_COOKIE_SAME_SITE_STRICT || same_site == U_COOKIE_SAME_SITE_LAX)) {
@@ -265,7 +237,7 @@ int ulfius_add_same_site_cookie_to_response(struct _u_response * response, const
         }
       }
     }
-    
+
     // Key not found, inserting a new cookie
     if (response->nb_cookies == 0) {
       response->map_cookie = o_malloc(sizeof(struct _u_cookie));
@@ -289,7 +261,7 @@ int ulfius_add_same_site_cookie_to_response(struct _u_response * response, const
     response->map_cookie[response->nb_cookies].secure = secure;
     response->map_cookie[response->nb_cookies].http_only = http_only;
     response->map_cookie[response->nb_cookies].same_site = same_site;
-    if ((key != NULL && response->map_cookie[response->nb_cookies].key == NULL) || (value != NULL && response->map_cookie[response->nb_cookies].value == NULL) || 
+    if ((key != NULL && response->map_cookie[response->nb_cookies].key == NULL) || (value != NULL && response->map_cookie[response->nb_cookies].value == NULL) ||
         (expires != NULL && response->map_cookie[response->nb_cookies].expires == NULL) || (domain != NULL && response->map_cookie[response->nb_cookies].domain == NULL) ||
         (path != NULL && response->map_cookie[response->nb_cookies].path == NULL)) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for ulfius_add_cookie_to_response");
@@ -308,11 +280,6 @@ int ulfius_add_same_site_cookie_to_response(struct _u_response * response, const
   }
 }
 
-/**
- * ulfius_clean_cookie
- * clean the cookie's elements
- * return U_OK on success
- */
 int ulfius_clean_cookie(struct _u_cookie * cookie) {
   if (cookie != NULL) {
     o_free(cookie->key);
@@ -331,10 +298,6 @@ int ulfius_clean_cookie(struct _u_cookie * cookie) {
   }
 }
 
-/**
- * Copy the cookie source elements into dest elements
- * return U_OK on success
- */
 int ulfius_copy_cookie(struct _u_cookie * dest, const struct _u_cookie * source) {
   if (source != NULL && dest != NULL) {
     dest->key = o_strdup(source->key);
@@ -360,13 +323,6 @@ int ulfius_copy_cookie(struct _u_cookie * dest, const struct _u_cookie * source)
   return U_ERROR_PARAMS;
 }
 
-/**
- * ulfius_clean_response
- * clean the specified response's inner elements
- * user must free the parent pointer if needed after clean
- * or use ulfius_clean_response_full
- * return U_OK on success
- */
 int ulfius_clean_response(struct _u_response * response) {
   unsigned int i;
   if (response != NULL) {
@@ -388,6 +344,8 @@ int ulfius_clean_response(struct _u_response * response) {
     if ((struct _websocket_handle *)response->websocket_handle) {
       o_free(((struct _websocket_handle *)response->websocket_handle)->websocket_protocol);
       o_free(((struct _websocket_handle *)response->websocket_handle)->websocket_extensions);
+      pointer_list_clean_free(((struct _websocket_handle *)response->websocket_handle)->websocket_extension_list, &ulfius_free_websocket_extension_pointer_list);
+      o_free(((struct _websocket_handle *)response->websocket_handle)->websocket_extension_list);
       o_free(response->websocket_handle);
     }
 #endif
@@ -397,11 +355,6 @@ int ulfius_clean_response(struct _u_response * response) {
   }
 }
 
-/**
- * ulfius_clean_response_full
- * clean the specified response and all its elements
- * return U_OK on success
- */
 int ulfius_clean_response_full(struct _u_response * response) {
   if (ulfius_clean_response(response) == U_OK) {
     o_free(response);
@@ -411,11 +364,6 @@ int ulfius_clean_response_full(struct _u_response * response) {
   }
 }
 
-/**
- * ulfius_init_response
- * Initialize a response structure by allocating inner elements
- * return U_OK on success
- */
 int ulfius_init_response(struct _u_response * response) {
   if (response != NULL) {
     response->status = 200;
@@ -434,9 +382,10 @@ int ulfius_init_response(struct _u_response * response) {
     response->binary_body = NULL;
     response->binary_body_length = 0;
     response->stream_callback = NULL;
-    response->stream_size = -1;
+    response->stream_size = U_STREAM_SIZE_UNKNOWN;
     response->stream_block_size = ULFIUS_STREAM_BLOCK_SIZE_DEFAULT;
     response->stream_callback_free = NULL;
+    response->stream_user_data = NULL;
     response->timeout = 0;
     response->shared_data = NULL;
 #ifndef U_DISABLE_WEBSOCKET
@@ -453,6 +402,13 @@ int ulfius_init_response(struct _u_response * response) {
     ((struct _websocket_handle *)response->websocket_handle)->websocket_incoming_user_data = NULL;
     ((struct _websocket_handle *)response->websocket_handle)->websocket_onclose_callback = NULL;
     ((struct _websocket_handle *)response->websocket_handle)->websocket_onclose_user_data = NULL;
+    ((struct _websocket_handle *)response->websocket_handle)->rsv_expected = 0;
+    if ((((struct _websocket_handle *)response->websocket_handle)->websocket_extension_list = o_malloc(sizeof(struct _pointer_list))) == NULL) {
+      o_free(response->websocket_handle);
+      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for response->websocket_handle->websocket_extension_list");
+      return U_ERROR_MEMORY;
+    }
+    pointer_list_init(((struct _websocket_handle *)response->websocket_handle)->websocket_extension_list);
 #endif
     return U_OK;
   } else {
@@ -460,86 +416,12 @@ int ulfius_init_response(struct _u_response * response) {
   }
 }
 
-/**
- * create a new response based on the source elements
- * return value must be free'd after use
- */
-struct _u_response * ulfius_duplicate_response(const struct _u_response * response) {
-  struct _u_response * new_response = NULL;
-  unsigned int i;
-  if (response != NULL) {
-    new_response = o_malloc(sizeof(struct _u_response));
-    if (new_response == NULL) {
-      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for new_response");
-      return NULL;
-    }
-    ulfius_init_response(new_response);
-    new_response->status = response->status;
-    new_response->protocol = o_strdup(response->protocol);
-    if (new_response->protocol == NULL) {
-      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for new_response->protocol");
-      ulfius_clean_response_full(new_response);
-      return NULL;
-    }
-    u_map_clean_full(new_response->map_header);
-    new_response->map_header = u_map_copy(response->map_header);
-    new_response->auth_realm = o_strdup(response->auth_realm);
-    new_response->nb_cookies = response->nb_cookies;
-    if (response->nb_cookies > 0) {
-      new_response->map_cookie = o_malloc(response->nb_cookies*sizeof(struct _u_cookie));
-      if (new_response->map_cookie == NULL) {
-        y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for new_response->map_cookie");
-        o_free(new_response);
-        return NULL;
-      }
-      for (i=0; i<response->nb_cookies; i++) {
-        ulfius_copy_cookie(&new_response->map_cookie[i], &response->map_cookie[i]);
-      }
-    } else {
-      new_response->map_cookie = NULL;
-    }
-    if (response->binary_body != NULL && response->binary_body_length > 0) {
-      new_response->binary_body = o_malloc(response->binary_body_length);
-      if (new_response->binary_body == NULL) {
-        y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for new_response->binary_body");
-        o_free(new_response->map_cookie);
-        o_free(new_response);
-        return NULL;
-      }
-      new_response->binary_body_length = response->binary_body_length;
-      memcpy(new_response->binary_body, response->binary_body, response->binary_body_length);
-    }
-#ifndef U_DISABLE_WEBSOCKET
-    if ((new_response->websocket_handle = o_malloc(sizeof(struct _websocket_handle))) != NULL) {
-      if (response->websocket_handle != NULL) {
-        ((struct _websocket_handle *)new_response->websocket_handle)->websocket_protocol = o_strdup(((struct _websocket_handle *)response->websocket_handle)->websocket_protocol);
-        ((struct _websocket_handle *)new_response->websocket_handle)->websocket_extensions = o_strdup(((struct _websocket_handle *)response->websocket_handle)->websocket_extensions);
-        ((struct _websocket_handle *)new_response->websocket_handle)->websocket_manager_callback = ((struct _websocket_handle *)response->websocket_handle)->websocket_manager_callback;
-        ((struct _websocket_handle *)new_response->websocket_handle)->websocket_manager_user_data = ((struct _websocket_handle *)response->websocket_handle)->websocket_manager_user_data;
-        ((struct _websocket_handle *)new_response->websocket_handle)->websocket_incoming_message_callback = ((struct _websocket_handle *)response->websocket_handle)->websocket_incoming_message_callback;
-        ((struct _websocket_handle *)new_response->websocket_handle)->websocket_incoming_user_data = ((struct _websocket_handle *)response->websocket_handle)->websocket_incoming_user_data;
-        ((struct _websocket_handle *)new_response->websocket_handle)->websocket_onclose_callback = ((struct _websocket_handle *)response->websocket_handle)->websocket_onclose_callback;
-        ((struct _websocket_handle *)new_response->websocket_handle)->websocket_onclose_user_data = ((struct _websocket_handle *)response->websocket_handle)->websocket_onclose_user_data;
-      }
-    } else {
-      ulfius_clean_response_full(new_response);
-      return NULL;
-    }
-#endif
-  }
-  return new_response;
-}
-
-/**
- * ulfius_copy_response
- * Copy the source response elements into the des response
- * return U_OK on success
- */
 int ulfius_copy_response(struct _u_response * dest, const struct _u_response * source) {
   unsigned int i;
   if (dest != NULL && source != NULL) {
     dest->status = source->status;
     dest->protocol = o_strdup(source->protocol);
+    dest->auth_realm = o_strdup(source->auth_realm);
     if (dest->protocol == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for dest->protocol");
       return U_ERROR_MEMORY;
@@ -562,7 +444,7 @@ int ulfius_copy_response(struct _u_response * dest, const struct _u_response * s
     } else {
       dest->map_cookie = NULL;
     }
-    
+
     if (source->binary_body != NULL && source->binary_body_length > 0) {
       dest->binary_body = o_malloc(source->binary_body_length);
       if (dest->binary_body == NULL) {
@@ -572,6 +454,17 @@ int ulfius_copy_response(struct _u_response * dest, const struct _u_response * s
       dest->binary_body_length = source->binary_body_length;
       memcpy(dest->binary_body, source->binary_body, source->binary_body_length);
     }
+
+    if (source->stream_callback != NULL) {
+      dest->stream_callback = source->stream_callback;
+      dest->stream_callback_free = source->stream_callback_free;
+      dest->stream_size = source->stream_size;
+      dest->stream_block_size = source->stream_block_size;
+      dest->stream_user_data = source->stream_user_data;
+    }
+
+    dest->shared_data = source->shared_data;
+    dest->timeout = source->timeout;
 #ifndef U_DISABLE_WEBSOCKET
     if (source->websocket_handle != NULL) {
       ((struct _websocket_handle *)dest->websocket_handle)->websocket_protocol = o_strdup(((struct _websocket_handle *)source->websocket_handle)->websocket_protocol);
@@ -590,26 +483,40 @@ int ulfius_copy_response(struct _u_response * dest, const struct _u_response * s
   }
 }
 
-/**
- * ulfius_set_string_body_response
- * Set a string binary_body to a response
- * binary_body must end with a '\0' character
- * return U_OK on success
- */
-int ulfius_set_string_body_response(struct _u_response * response, const unsigned int status, const char * binary_body) {
-  if (response != NULL && binary_body != NULL) {
-    size_t binary_body_length = o_strlen(binary_body);
-    // Free the binary_body available
+struct _u_response * ulfius_duplicate_response(const struct _u_response * response) {
+  struct _u_response * new_response = NULL;
+  if (response != NULL) {
+    new_response = o_malloc(sizeof(struct _u_response));
+    if (new_response == NULL) {
+      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for new_response");
+      return NULL;
+    }
+    if (ulfius_init_response(new_response) == U_OK) {
+      if (ulfius_copy_response(new_response, response) != U_OK) {
+        y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error ulfius_copy_response");
+        ulfius_clean_response_full(new_response);
+        new_response = NULL;
+      }
+    } else {
+      y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error ulfius_init_response");
+      o_free(new_response);
+      new_response = NULL;
+    }
+  }
+  return new_response;
+}
+
+int ulfius_set_string_body_response(struct _u_response * response, const unsigned int status, const char * string_body) {
+  if (response != NULL && string_body != NULL) {
+    // Free all the bodies available
     o_free(response->binary_body);
-    response->binary_body = o_malloc(binary_body_length);
-    
+    response->binary_body = o_strdup(string_body);
     if (response->binary_body == NULL) {
       y_log_message(Y_LOG_LEVEL_ERROR, "Ulfius - Error allocating memory for response->binary_body");
       return U_ERROR_MEMORY;
     } else {
       response->status = status;
-      response->binary_body_length = binary_body_length;
-      memcpy(response->binary_body, binary_body, binary_body_length);
+      response->binary_body_length = o_strlen(string_body);
       return U_OK;
     }
   } else {
@@ -617,11 +524,6 @@ int ulfius_set_string_body_response(struct _u_response * response, const unsigne
   }
 }
 
-/**
- * ulfius_set_binary_body_response
- * Add a binary binary_body to a response
- * return U_OK on success
- */
 int ulfius_set_binary_body_response(struct _u_response * response, const unsigned int status, const char * binary_body, const size_t length) {
   if (response != NULL && binary_body != NULL && length > 0) {
     // Free all the bodies available
@@ -643,17 +545,13 @@ int ulfius_set_binary_body_response(struct _u_response * response, const unsigne
   }
 }
 
-/**
- * ulfius_set_empty_body_response
- * Set an empty response with only a status
- * return U_OK on success
- */
 int ulfius_set_empty_body_response(struct _u_response * response, const unsigned int status) {
   if (response != NULL) {
     // Free all the bodies available
+    o_free(response->binary_body);
     response->binary_body = NULL;
     response->binary_body_length = 0;
-    
+
     response->status = status;
     return U_OK;
   } else {
@@ -661,14 +559,7 @@ int ulfius_set_empty_body_response(struct _u_response * response, const unsigned
   }
 }
 
-/**
- * ulfius_set_stream_response
- * Set an stream response with a status
- * set stream_size to -1 if unknown
- * set stream_block_size to a proper value based on the system
- * return U_OK on success
- */
-int ulfius_set_stream_response(struct _u_response * response, 
+int ulfius_set_stream_response(struct _u_response * response,
                                 const unsigned int status,
                                 ssize_t (* stream_callback) (void * stream_user_data, uint64_t offset, char * out_buf, size_t max),
                                 void (* stream_callback_free) (void * stream_user_data),
@@ -680,7 +571,7 @@ int ulfius_set_stream_response(struct _u_response * response,
     o_free(response->binary_body);
     response->binary_body = NULL;
     response->binary_body_length = 0;
-    
+
     response->status = status;
     response->stream_callback = stream_callback;
     response->stream_callback_free = stream_callback_free;
@@ -693,6 +584,75 @@ int ulfius_set_stream_response(struct _u_response * response,
   }
 }
 
+int ulfius_set_response_properties(struct _u_response * response, ...) {
+  u_option option;
+  int ret = U_OK;
+  const char * str_key, * str_value;
+  size_t size_value;
+#ifndef U_DISABLE_JANSSON
+  json_t * j_value;
+#endif
+  va_list vl;
+
+  if (response != NULL) {
+    va_start(vl, response);
+    for (option = va_arg(vl, uint); option != U_OPT_NONE && ret == U_OK; option = va_arg(vl, uint)) {
+      switch (option) {
+        case U_OPT_STATUS:
+          response->status = va_arg(vl, long);
+          break;
+        case U_OPT_HEADER_PARAMETER:
+          str_key = va_arg(vl, const char *);
+          str_value = va_arg(vl, const char *);
+          ret = u_map_put(response->map_header, str_key, str_value);
+          break;
+        case U_OPT_HEADER_PARAMETER_REMOVE:
+          str_key = va_arg(vl, const char *);
+          ret = u_map_remove_from_key(response->map_header, str_key);
+          break;
+        case U_OPT_AUTH_REALM:
+          str_value = va_arg(vl, const char *);
+          o_free(response->auth_realm);
+          if (o_strlen(str_value)) {
+            response->auth_realm = o_strdup(str_value);
+          } else {
+            response->auth_realm = NULL;
+          }
+          break;
+        case U_OPT_BINARY_BODY:
+          str_value = va_arg(vl, const char *);
+          size_value = va_arg(vl, size_t);
+          ret = ulfius_set_binary_body_response(response, response->status, str_value, size_value);
+          break;
+        case U_OPT_STRING_BODY:
+          str_value = va_arg(vl, const char *);
+          ret = ulfius_set_string_body_response(response, response->status, str_value);
+          break;
+#ifndef U_DISABLE_JANSSON
+        case U_OPT_JSON_BODY:
+          j_value = va_arg(vl, json_t *);
+          ret = ulfius_set_json_body_response(response, response->status, j_value);
+          break;
+#endif
+        case U_OPT_SHARED_DATA:
+          response->shared_data = va_arg(vl, void *);
+          break;
+        case U_OPT_TIMEOUT:
+          response->timeout = va_arg(vl, unsigned int);
+          break;
+        default:
+          ret = U_ERROR_PARAMS;
+          break;
+      }
+    }
+    va_end(vl);
+  } else {
+    y_log_message(Y_LOG_LEVEL_DEBUG, "Ulfius - Error input parameter");
+    ret = U_ERROR_PARAMS;
+  }
+  return ret;
+}
+
 #ifndef U_DISABLE_JANSSON
 /**
  * ulfius_set_json_body_response
@@ -734,11 +694,6 @@ json_t * ulfius_get_json_body_response(struct _u_response * response, json_error
 }
 #endif
 
-/**
- * ulfius_add_header_to_response
- * add a header to the response
- * return U_OK on success
- */
 int ulfius_add_header_to_response(struct _u_response * response, const char * key, const char * value) {
   if (response != NULL && key != NULL && value != NULL) {
     return u_map_put(response->map_header, key, value);
