@@ -23,6 +23,7 @@ START_TEST(test_u_map_put)
   struct _u_map map;
   u_map_init(&map);
   ck_assert_int_eq(u_map_put(&map, "key", "value"), U_OK);
+  ck_assert_int_eq(u_map_put(&map, "key", "value2"), U_OK);
   ck_assert_int_eq(u_map_put(&map, "key", NULL), U_OK);
   ck_assert_int_eq(u_map_put(&map, NULL, "value"), U_ERROR_PARAMS);
   ck_assert_int_eq(u_map_put(&map, NULL, NULL), U_ERROR_PARAMS);
@@ -34,6 +35,7 @@ START_TEST(test_u_map_put)
   ck_assert_int_eq(u_map_put_binary(&map, NULL, NULL, 0, o_strlen("value")), U_ERROR_PARAMS);
   ck_assert_int_eq(u_map_put_binary(NULL, "bkey", "value", 0, o_strlen("value")), U_ERROR_PARAMS);
   ck_assert_int_eq(u_map_put_binary(&map, "bkey", "new_value", 0, o_strlen("new_value")), U_OK);
+  ck_assert_int_eq(u_map_put(&map, "key", "value"), U_OK);
   u_map_clean(&map);
 }
 END_TEST
@@ -55,6 +57,16 @@ START_TEST(test_u_map_get)
   ck_assert_ptr_eq((void *)u_map_get_case(NULL, "keY"), NULL);
   ck_assert_int_eq(u_map_get_case_length(&map, "noPe"), -1);
   ck_assert_int_eq(u_map_get_case_length(NULL, "KEy"), -1);
+  ck_assert_int_eq(u_map_put(&map, "key", NULL), U_OK);
+  ck_assert_ptr_eq(u_map_get(&map, "key"), NULL);
+  ck_assert_int_eq(u_map_put(&map, "key", "value"), U_OK);
+  ck_assert_str_eq(u_map_get(&map, "key"), "value");
+  ck_assert_int_eq(u_map_put(&map, "key", NULL), U_OK);
+  ck_assert_ptr_eq(u_map_get(&map, "key"), NULL);
+  ck_assert_int_eq(u_map_put(&map, "key", "value"), U_OK);
+  ck_assert_str_eq(u_map_get(&map, "key"), "value");
+  ck_assert_int_eq(u_map_put(&map, "key", NULL), U_OK);
+  ck_assert_ptr_eq(u_map_get(&map, "key"), NULL);
   u_map_clean(&map);
 }
 END_TEST
@@ -191,6 +203,8 @@ int main(int argc, char *argv[])
   Suite *s;
   SRunner *sr;
   
+  ulfius_global_init();
+  
   s = ulfius_suite();
   sr = srunner_create(s);
 
@@ -198,5 +212,6 @@ int main(int argc, char *argv[])
   number_failed = srunner_ntests_failed(sr);
   srunner_free(sr);
   
-	return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
+  ulfius_global_close();
+  return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
